@@ -4,7 +4,6 @@ use strict;
 use warnings;
 
 use base 'Object::Accessor';
-
 use base 'CPANPLUS::Internals::Utils';
 
 use Config;
@@ -20,6 +19,8 @@ use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 use Module::Load::Conditional   qw[check_install];
 use version;
 
+use vars qw[$VERSION];
+$VERSION = "0.9134";
 
 =pod
 
@@ -166,7 +167,10 @@ Defaults to C<true>.
 =item base
 
 The directory CPANPLUS keeps all its build and state information in.
-Defaults to ~/.cpanplus.
+Defaults to ~/.cpanplus. If L<File::HomeDir> is available, that will
+be used to work out your C<HOME> directory. This may be overriden by
+setting the C<PERL5_CPANPLUS_HOME> environment variable, see
+L<CPANPLUS::Config::HomeEnv> for more details.
 
 =cut
 
