@@ -20,18 +20,18 @@ local $Params::Check::VERBOSE = 1;
 
 ### require, avoid circular use ###
 require CPANPLUS::Internals;
-$VERSION = $CPANPLUS::Internals::VERSION = $CPANPLUS::Internals::VERSION;
+$VERSION = "0.9134";
 
 ### can't use O::A as we're using our own AUTOLOAD to get to
 ### the config options.
-for my $meth ( qw[conf]) {
+for my $meth ( qw[conf _lib _perl5lib]) {
     no strict 'refs';
-    
+
     *$meth = sub {
         my $self = shift;
         $self->{'_'.$meth} = $_[0] if @_;
         return $self->{'_'.$meth};
-    }     
+    }
 }
 
 
@@ -39,7 +39,7 @@ for my $meth ( qw[conf]) {
 
 =head1 NAME
 
-CPANPLUS::Configure
+CPANPLUS::Configure - configuration for CPANPLUS
 
 =head1 SYNOPSIS
 
@@ -70,12 +70,18 @@ This method returns a new object. Normal users will never need to
 invoke the C<new> method, but instead retrieve the desired object via
 a method call on a C<CPANPLUS::Backend> object.
 
-The C<load_configs> parameter controls wether or not additional
-user configurations are to be loaded or not. Defaults to C<true>.
+=over 4
+
+=item load_configs
+
+Controls whether or not additional user configurations are to be loaded
+or not. Defaults to C<true>.
+
+=back
 
 =cut
 
-### store teh CPANPLUS::Config object in a closure, so we only
+### store the CPANPLUS::Config object in a closure, so we only
 ### initialize it once.. otherwise, on a 2nd ->new, settings
 ### from configs on top of this one will be reset
 {   my $Config;
@@ -83,25 +89,30 @@ user configurations are to be loaded or not. Defaults to C<true>.
     sub new {
         my $class   = shift;
         my %hash    = @_;
-        
+
         ### XXX pass on options to ->init() like rescan?
         my ($load);
         my $tmpl    = {
             load_configs    => { default => 1, store => \$load },
         };
-        
+
         check( $tmpl, \%hash ) or (
             warn Params::Check->last_error, return
         );
-        
+
         $Config     ||= CPANPLUS::Config->new;
         my $self    = bless {}, $class;
         $self->conf( $Config );
-    
+
         ### you want us to load other configs?
         ### these can override things in the default config
         $self->init if $load;
-    
+
+        ### after processing the config files, check what
+        ### @INC and PERL5LIB are set to.
+        $self->_lib( \@INC );
+        $self->_perl5lib( $ENV{'PERL5LIB'} );
+
         return $self;
     }
 }
@@ -132,16 +143,21 @@ Returns true on success, false on failure.
         my $self    = shift;
         my $obj     = $self->conf;
         my %hash    = @_;
-        
+
         my ($rescan);
         my $tmpl    = {
             rescan  => { default => 0, store => \$rescan },
         };
-        
+
         check( $tmpl, \%hash ) or (
             warn Params::Check->last_error, return
-        );        
-        
+        );
+
+        ### if the base dir is changed, we have to rescan it
+        ### for any CPANPLUS::Config::* files as well, so keep
+        ### track of it
+        my $cur_base = $self->get_conf('base');
+
         ### warn if we find an old style config specified
         ### via environment variables
         {   my $env = ENV_CPANPLUS_CONFIG;
@@ -153,63 +169,85 @@ Returns true on success, false on failure.
                           "in the default shell to use custom config files.",
                           $env, "CPANPLUS::Configure->save", 's save'));
             }
-        }            
-        
-        ### make sure that the homedir is included now
-        local @INC = ( CONFIG_USER_LIB_DIR->(), @INC );
-        
-        ### only set it up once
-        if( !$loaded++ or $rescan ) {   
-            ### find plugins & extra configs
-            ### check $home/.cpanplus/lib as well
-            require Module::Pluggable;
-            
-            Module::Pluggable->import(
-                search_path => ['CPANPLUS::Config'],
-                search_dirs => [ CONFIG_USER_LIB_DIR ],
-                except      => qr/::SUPER$/,
-                sub_name    => 'configs'
-            );
         }
-        
-        
-        ### do system config, user config, rest.. in that order
-        ### apparently, on a 2nd invocation of -->configs, a
-        ### ::ISA::CACHE package can appear.. that's bad...
-        my %confs = map  { $_ => $_ } 
-                    grep { $_ !~ /::ISA::/ } __PACKAGE__->configs;
-        my @confs = grep { defined } 
-                    map  { delete $confs{$_} } CONFIG_SYSTEM, CONFIG_USER;
-        push @confs, sort keys %confs;                    
-    
-        for my $plugin ( @confs ) {
-            msg(loc("Found config '%1'", $plugin),0);
-            
-            ### if we already did this the /last/ time around dont 
-            ### run the setup agian.
-            if( my $loc = Module::Loaded::is_loaded( $plugin ) ) {
-                msg(loc("  Already loaded '%1' (%2)", $plugin, $loc), 0);
-                next;
-            } else {
-                msg(loc("  Loading config '%1'", $plugin),0);
-            
-                eval { load $plugin };
-                msg(loc("  Loaded '%1' (%2)", 
-                        $plugin, Module::Loaded::is_loaded( $plugin ) ), 0);
-            }                   
-            
-            if( $@ ) {
-                error(loc("Could not load '%1': %2", $plugin, $@));
-                next;
-            }     
-            
-            my $sub = $plugin->can('setup');
-            $sub->( $self ) if $sub;
+
+        {   ### make sure that the homedir is included now
+            local @INC = ( LIB_DIR->($cur_base), @INC );
+
+            ### only set it up once
+            if( !$loaded++ or $rescan ) {
+                ### find plugins & extra configs
+                ### check $home/.cpanplus/lib as well
+                require Module::Pluggable;
+
+                Module::Pluggable->import(
+                    search_path => ['CPANPLUS::Config'],
+                    search_dirs => [ LIB_DIR->($cur_base) ],
+                    except      => qr/::SUPER$/,
+                    sub_name    => 'configs'
+                );
+            }
+
+
+            ### do system config, user config, rest.. in that order
+            ### apparently, on a 2nd invocation of -->configs, a
+            ### ::ISA::CACHE package can appear.. that's bad...
+            my %confs = map  { $_ => $_ }
+                        grep { $_ !~ /::ISA::/ } __PACKAGE__->configs;
+            my @confs = grep { defined }
+                        map  { delete $confs{$_} } CONFIG_SYSTEM, CONFIG_USER;
+            push @confs, sort keys %confs;
+
+            for my $plugin ( @confs ) {
+                msg(loc("Found config '%1'", $plugin),0);
+
+                ### if we already did this the /last/ time around dont
+                ### run the setup agian.
+                if( my $loc = Module::Loaded::is_loaded( $plugin ) ) {
+                    msg(loc("  Already loaded '%1' (%2)", $plugin, $loc), 0);
+                    next;
+                } else {
+                    msg(loc("  Loading config '%1'", $plugin),0);
+
+                    if( eval { load $plugin; 1 } ) {
+                        msg(loc("  Loaded '%1' (%2)",
+                            $plugin, Module::Loaded::is_loaded( $plugin ) ), 0);
+                    } else {
+                        error(loc("  Error loading '%1': %2", $plugin, $@));
+                    }
+                }
+
+                if( $@ ) {
+                    error(loc("Could not load '%1': %2", $plugin, $@));
+                    next;
+                }
+
+                my $sub = $plugin->can('setup');
+                $sub->( $self ) if $sub;
+            }
         }
-        
+
+        ### did one of the plugins change the base dir? then we should
+        ### scan the dirs again
+        if( $cur_base ne $self->get_conf('base') ) {
+            msg(loc("Base dir changed from '%1' to '%2', rescanning",
+                    $cur_base, $self->get_conf('base')), 0);
+            $self->init( @_, rescan => 1 );
+        }
+
         ### clean up the paths once more, just in case
         $obj->_clean_up_paths;
-    
+
+        ### XXX in case the 'lib' param got changed, we need to
+        ### add that now, or it's not propagating ;(
+        {   my $lib = $self->get_conf('lib');
+            my %inc = map { $_ => $_ } @INC;
+            for my $l ( @$lib ) {
+                push @INC, $l unless $inc{$l};
+            }
+            $self->_lib( \@INC );
+        }
+
         return 1;
     }
 }
@@ -227,7 +265,7 @@ Returns true if the file can be saved, false otherwise.
 sub can_save {
     my $self = shift;
     my $file = shift || CONFIG_USER_FILE->();
-    
+
     return 1 unless -e $file;
 
     chmod 0644, $file;
@@ -246,7 +284,7 @@ be attempted to be saved in the system wide directory.
 If no argument is provided, it will default to your personal
 config.
 
-Returns the full path to the file if the config was saved, 
+Returns the full path to the file if the config was saved,
 false otherwise.
 
 =cut
@@ -260,12 +298,12 @@ sub _config_pm_to_file {
     ### so figure out where to save them based on their type
     my $file;
     if( $pm eq CONFIG_USER ) {
-        $file = CONFIG_USER_FILE->();   
+        $file = CONFIG_USER_FILE->();
 
     } elsif ( $pm eq CONFIG_SYSTEM ) {
         $file = CONFIG_SYSTEM_FILE->();
-        
-    ### third party file        
+
+    ### third party file
     } else {
         my $cfg_pkg = CONFIG . '::';
         unless( $pm =~ /^$cfg_pkg/ ) {
@@ -273,13 +311,13 @@ sub _config_pm_to_file {
                 "WARNING: Your config package '%1' is not in the '%2' ".
                 "namespace and will not be automatically detected by %3",
                 $pm, $cfg_pkg, 'CPANPLUS'
-            ));        
-        }                        
-    
+            ));
+        }
+
         $file = File::Spec->catfile(
             $dir,
             split( '::', $pm )
-        ) . '.pm';        
+        ) . '.pm';
     }
 
     return $file;
@@ -290,35 +328,35 @@ sub save {
     my $self    = shift;
     my $pm      = shift || CONFIG_USER;
     my $savedir = shift || '';
-    
+
     my $file = $self->_config_pm_to_file( $pm, $savedir ) or return;
     my $dir  = dirname( $file );
-    
+
     unless( -d $dir ) {
         $self->_mkdir( dir => $dir ) or (
             error(loc("Can not create directory '%1' to save config to",$dir)),
             return
         )
-    }       
+    }
     return unless $self->can_save($file);
 
-    ### find only accesors that are not private
+    ### find only accessors that are not private
     my @acc = sort grep { $_ !~ /^_/ } $self->conf->ls_accessors;
 
     ### for dumping the values
     use Data::Dumper;
-    
+
     my @lines;
     for my $acc ( @acc ) {
-        
+
         push @lines, "### $acc section", $/;
-        
+
         for my $key ( $self->conf->$acc->ls_accessors ) {
             my $val = Dumper( $self->conf->$acc->$key );
-        
+
             $val =~ s/\$VAR1\s+=\s+//;
             $val =~ s/;\n//;
-        
+
             push @lines, '$'. "conf->set_${acc}( $key => $val );", $/;
         }
         push @lines, $/,$/;
@@ -330,13 +368,13 @@ sub save {
     ### use a variable to make sure the pod parser doesn't snag it
     my $is      = '=';
     my $time    = gmtime;
-   
-    
+
+
     my $msg     = <<_END_OF_CONFIG_;
 ###############################################
-###                                         
-###  Configuration structure for $pm        
-###                                         
+###
+###  Configuration structure for $pm
+###
 ###############################################
 
 #last changed: $time GMT
@@ -361,11 +399,11 @@ use strict;
 
 sub setup {
     my \$conf = shift;
-    
+
 $str
 
-    return 1;    
-} 
+    return 1;
+}
 
 1;
 
@@ -409,7 +447,7 @@ sub options {
 
     my %seen;
     return sort grep { !$seen{$_}++ }
-                map { $_->$type->ls_accessors if $_->can($type)  } 
+                map { $_->$type->ls_accessors if $_->can($type)  }
                 $self->conf;
     return;
 }
@@ -421,6 +459,9 @@ sub options {
 Accessors that start with a C<_> are marked private -- regular users
 should never need to use these.
 
+See the C<CPANPLUS::Config> documentation for what items can be
+set and retrieved.
+
 =head2 get_SOMETHING( ITEM, [ITEM, ITEM, ... ] );
 
 The C<get_*> style accessors merely retrieves one or more desired
@@ -460,18 +501,10 @@ Locations and names of source files locally.
 
 Locations and names of source files remotely.
 
-=item _set|_get_dist
-
-Mapping of distribution format names to modules.
-
 =item _set|_get_fetch
 
 Special settings pertaining to the fetching of files.
 
-=item _set|_get_daemon
-
-Settings for C<cpanpd>, the CPANPLUS daemon.
-
 =back
 
 =cut
@@ -490,10 +523,12 @@ sub AUTOLOAD {
     $type .= '_'    if $private;
     $type .= $field if $field;
 
-    unless ( $conf->can($type) ) {
+    my $type_code = $conf->can($type);
+    unless ( $type_code ) {
         error( loc("Invalid method type: '%1'", $name) );
         return;
     }
+    my $type_obj = $type_code->();
 
     unless( scalar @_ ) {
         error( loc("No arguments provided!") );
@@ -506,16 +541,16 @@ sub AUTOLOAD {
             my @list = ();
 
             ### get it from the user config first
-            if( $conf->can($type) and $conf->$type->can($key) ) {
-                push @list, $conf->$type->$key;
+            if( my $code = $type_obj->can($key) ) {
+                push @list, $code->();
 
             ### XXX EU::AI compatibility hack to provide lookups like in
             ### cpanplus 0.04x; we renamed ->_get_build('base') to
             ### ->get_conf('base')
             } elsif ( $type eq '_build' and $key eq 'base' ) {
-                return $self->get_conf($key);  
-                
-            } else {     
+                return $self->get_conf($key);
+
+            } else {
                 error( loc(q[No such key '%1' in field '%2'], $key, $type) );
                 return;
             }
@@ -529,9 +564,9 @@ sub AUTOLOAD {
 
         while( my($key,$val) = each %args ) {
 
-            if( $conf->can($type) and $conf->$type->can($key) ) {
-                $conf->$type->$key( $val );
-                
+            if( my $code = $type_obj->can($key) ) {
+                $code->( $val );
+
             } else {
                 error( loc(q[No such key '%1' in field '%2'], $key, $type) );
                 return;
@@ -546,13 +581,13 @@ sub AUTOLOAD {
 
         while( my($key,$val) = each %args ) {
 
-            if( $conf->$type->can($key) ) {
+            if( $type_obj->can($key) ) {
                 error( loc( q[Key '%1' already exists for field '%2'],
                             $key, $type));
                 return;
             } else {
-                $conf->$type->mk_accessors( $key );
-                $conf->$type->$key( $val );
+                $type_obj->mk_accessors( $key );
+                $type_obj->$key( $val );
             }
         }
         return 1;
@@ -580,15 +615,15 @@ This module by Jos Boumans E<lt>kane@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-The CPAN++ interface (of which this module is a part of) is copyright (c) 
+The CPAN++ interface (of which this module is a part of) is copyright (c)
 2001 - 2007, Jos Boumans E<lt>kane@cpan.orgE<gt>. All rights reserved.
 
-This library is free software; you may redistribute and/or modify it 
+This library is free software; you may redistribute and/or modify it
 under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<CPANPLUS::Backend>, L<CPANPLUS::Configure::Setup>
+L<CPANPLUS::Backend>, L<CPANPLUS::Configure::Setup>, L<CPANPLUS::Config>
 
 =cut
 
