@@ -14,6 +14,9 @@ use Params::Check               qw[check];
 use Module::Load::Conditional   qw[can_load check_install];
 use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 
+use vars qw[$VERSION];
+$VERSION = "0.9134";
+
 use base 'Object::Accessor';
 
 local $Params::Check::VERBOSE = 1;
@@ -22,7 +25,7 @@ local $Params::Check::VERBOSE = 1;
 
 =head1 NAME
 
-CPANPLUS::Dist
+CPANPLUS::Dist - base class for plugins
 
 =head1 SYNOPSIS
 
@@ -610,11 +613,7 @@ sub _resolve_prereqs {
         ### 'perl' is a special case, there's no mod object for it
         if( $mod eq PERL_CORE ) {
 
-            ### run a CLI invocation to see if the perl you specified is
-            ### uptodate
-            my $ok = run( command => "$^X -M$version -e1", verbose => 0 );
-
-            unless( $ok ) {
+            unless( $cb->_vcmp( sprintf('v%vd',$^V), $version ) >= 0 ) {
                 error(loc(  "Module '%1' needs perl version '%2', but you ".
                             "only have version '%3' -- can not proceed",
                             $self->module, $version,
