@@ -1,9 +1,10 @@
 package CPANPLUS::Dist::MM;
 
-use warnings;
 use strict;
-use vars    qw[@ISA $STATUS];
+use warnings;
+use vars    qw[@ISA $STATUS $VERSION];
 use base    'CPANPLUS::Dist::Base';
+$VERSION = "0.9134";
 
 use CPANPLUS::Internals::Constants;
 use CPANPLUS::Internals::Constants::Report;
@@ -23,7 +24,7 @@ local $Params::Check::VERBOSE = 1;
 
 =head1 NAME
 
-CPANPLUS::Dist::MM
+CPANPLUS::Dist::MM - distribution class for MakeMaker related modules
 
 =head1 SYNOPSIS
 
@@ -221,14 +222,14 @@ sub prepare {
     }
 
     my $args;
-    my( $force, $verbose, $perl, @mmflags, $prereq_target, $prereq_format,
+    my( $force, $verbose, $perl, $mmflags, $prereq_target, $prereq_format,
         $prereq_build );
     {   local $Params::Check::ALLOW_UNKNOWN = 1;
         my $tmpl = {
             perl            => {    default => $^X, store => \$perl },
             makemakerflags  => {    default =>
                                         $conf->get_conf('makemakerflags') || '',
-                                    store => \$mmflags[0] },
+                                    store => \$mmflags },
             force           => {    default => $conf->get_conf('force'),
                                     store   => \$force },
             verbose         => {    default => $conf->get_conf('verbose'),
@@ -242,6 +243,7 @@ sub prepare {
         $args = check( $tmpl, \%hash ) or return;
     }
 
+    my @mmflags = $dist->_split_like_shell( $mmflags );
 
     ### maybe we already ran a create on this object? ###
     return 1 if $dist->status->prepared && !$force;
@@ -353,20 +355,8 @@ sub prepare {
             ### in @INC, stopping us from resolving dependencies on CPANPLUS
             ### at bootstrap time properly.
 
-            ### XXX this fails under ipc::run due to the extra quotes,
-            ### but it works in ipc::open3. however, ipc::open3 doesn't work
-            ### on win32/cygwin. XXX TODO get a windows box and sort this out
-            # my $cmd =  qq[$perl -MEnglish -le ] .
-            #            QUOTE_PERL_ONE_LINER->(
-            #                qq[\$OUTPUT_AUTOFLUSH++,do(q($makefile_pl))]
-            #            )
-            #            . $mmflags;
-
-            # my $flush = OPT_AUTOFLUSH;
-            # my $cmd     = "$perl $flush $makefile_pl $mmflags";
-
-            my $run_perl    = $conf->get_program('perlwrapper');
-            my $cmd         = [$perl, $run_perl, $makefile_pl, @mmflags];
+            my @run_perl    = ( '-e', PERL_WRAPPER );
+            my $cmd         = [$perl, @run_perl, $makefile_pl, @mmflags];
 
             ### set ENV var to tell underlying code this is what we're
             ### executing.
@@ -418,8 +408,12 @@ sub prepare {
         ### Make (haha) sure that Makefile.PL is older than the Makefile
         ### we just generated.
         eval {
-          my $ftime = time - 4;
-          utime $ftime, $ftime, MAKEFILE_PL->( $cb->_safe_path( path => $dir ) );
+          my $makestat = ( stat MAKEFILE->( $dir ) )[9];
+          my $mplstat = ( stat MAKEFILE_PL->( $cb->_safe_path( path => $dir ) ) )[9];
+          if ( $makestat < $mplstat ) {
+            my $ftime = $makestat - 60;
+            utime $ftime, $ftime, MAKEFILE_PL->( $cb->_safe_path( path => $dir ) );
+          }
         };
 
         ### start resolving prereqs ###
@@ -586,6 +580,8 @@ sub create {
         $args = check( $tmpl, \%hash ) or return;
     }
 
+    my @makeflags = $dist->_split_like_shell( $makeflags );
+
     ### maybe we already ran a create on this object?
     ### make sure we add to include path again, just in case we came from
     ### ->save_state, at which point we need to restore @INC/$PERL5LIB
@@ -612,6 +608,7 @@ sub create {
     }
 
     my $fail; my $prereq_fail; my $test_fail;
+    my $status = { };
     RUN: {
         ### this will set the directory back to the start
         ### dir, so we must chdir /again/
@@ -648,11 +645,15 @@ sub create {
                     "not running again unless you force",
                     $make, $self->module ), $verbose );
         } else {
-            unless(scalar run(  command => [$make, $makeflags],
+            unless(scalar run(  command => [$make, @makeflags],
                                 buffer  => \$captured,
                                 verbose => $verbose )
             ) {
                 error( loc( "MAKE failed: %1 %2", $!, $captured ) );
+                if ( $conf->get_conf('cpantest') ) {
+                  $status->{stage} = 'build';
+                  $status->{capture} = $captured;
+                }
                 $dist->status->make(0);
                 $fail++; last RUN;
             }
@@ -690,7 +691,7 @@ sub create {
             ### XXX need to add makeflags here too?
             ### yes, but they should really be split out -- see bug #4143
             if( scalar run(
-                        command => [$make, 'test', $makeflags],
+                        command => [$make, 'test', @makeflags],
                         buffer  => \$captured,
                         verbose => $run_verbose,
             ) ) {
@@ -703,10 +704,20 @@ sub create {
                     msg( loc( "MAKE TEST passed: %1", $captured ), 0 );
                 }
 
+                if ( $conf->get_conf('cpantest') ) {
+                  $status->{stage} = 'test';
+                  $status->{capture} = $captured;
+                }
+
                 $dist->status->test(1);
             } else {
                 error( loc( "MAKE TEST failed: %1", $captured ), ( $run_verbose ? 0 : 1 ) );
 
+                if ( $conf->get_conf('cpantest') ) {
+                  $status->{stage} = 'test';
+                  $status->{capture} = $captured;
+                }
+
                 ### send out error report here? or do so at a higher level?
                 ### --higher level --kane.
                 $dist->status->test(0);
@@ -728,6 +739,7 @@ sub create {
         error( loc( "Could not chdir back to start dir '%1'", $orig ) );
     }
 
+    ### TODO: Add $stage to _send_report()
     ### send out test report?
     ### only do so if the failure is this module, not its prereq
     if( $conf->get_conf('cpantest') and not $prereq_fail) {
@@ -735,6 +747,7 @@ sub create {
             module  => $self,
             failed  => $test_fail || $fail,
             buffer  => CPANPLUS::Error->stack_as_string,
+            status  => $status,
             verbose => $verbose,
             force   => $force,
         ) or error(loc("Failed to send test report for '%1'",
@@ -806,6 +819,7 @@ sub install {
         return;
     }
 
+    my @makeflags = $dist->_split_like_shell( $makeflags );
 
     $dist->status->_install_args( $args );
 
@@ -820,7 +834,7 @@ sub install {
     ### 'make install' section ###
     ### XXX need makeflags here too?
     ### yes, but they should really be split out.. see bug #4143
-    my $cmd     = [$make, 'install', $makeflags];
+    my $cmd     = [$make, 'install', @makeflags];
     my $sudo    = $conf->get_program('sudo');
     unshift @$cmd, $sudo if $sudo and $>;
 
@@ -1007,6 +1021,17 @@ sub dist_dir {
     return $distdir;
 }
 
+sub _split_like_shell {
+  my ($self, $string) = @_;
+
+  return () unless defined($string);
+  return @$string if ref $string eq 'ARRAY';
+  $string =~ s/^\s+|\s+$//g;
+  return () unless length($string);
+
+  require Text::ParseWords;
+  return Text::ParseWords::shellwords($string);
+}
 
 1;
 
