@@ -1,6 +1,8 @@
 package CPANPLUS::Error;
 
 use strict;
+use vars qw[$VERSION];
+$VERSION = "0.9134";
 
 use Log::Message private => 0;;
 
@@ -8,7 +10,7 @@ use Log::Message private => 0;;
 
 =head1 NAME
 
-CPANPLUS::Error
+CPANPLUS::Error - error handling for CPANPLUS
 
 =head1 SYNOPSIS
 
