@@ -11,14 +11,13 @@ use vars    qw[$VERSION @ISA @EXPORT];
 
 use Package::Constants;
 
-
-$VERSION    = 0.01;
-@ISA        = qw[Exporter];
-@EXPORT     = Package::Constants->list( __PACKAGE__ );
-
 ### for the version
 require CPANPLUS::Internals;
 
+$VERSION = "0.9134";
+@ISA        = qw[Exporter];
+@EXPORT     = Package::Constants->list( __PACKAGE__ );
+
 ### OS to regex map ###
 my %OS = (
     Amiga       => 'amigaos',
@@ -80,10 +79,10 @@ use constant RELEVANT_TEST_RESULT
                                 my $name = $mod->module;
                                 my $specific;
                                 for my $platform (keys %OS) {
-                                    if( $name =~ /\b$platform\b/i ) {
+                                    if( $name =~ /^$platform\b/i ) {
                                         # beware the Mac != MAC
                                         next if($platform eq 'Mac' &&
-                                                $name !~ /\b$platform\b/);
+                                                $name !~ /^$platform\b/);
                                         $specific++;
                                         return 1 if
                                             $^O =~ /^(?:$OS{$platform})$/
@@ -100,7 +99,7 @@ use constant UNSUPPORTED_OS
                                     return 1;
                                 }
                                 return 0;
-                          };                                            
+                          };
 
 use constant PERL_VERSION_TOO_LOW
                             => sub {
@@ -116,7 +115,7 @@ use constant PERL_VERSION_TOO_LOW
                                     return 1;
                                 }
                                 return 0;
-                          };                                            
+                          };
 
 use constant NO_TESTS_DEFINED
                             => sub {
@@ -125,10 +124,10 @@ use constant NO_TESTS_DEFINED
                                   /(No tests defined( for [\w:]+ extension)?\.)/
                                   and $buffer !~ /\*\.t/m and
                                       $buffer !~ /test\.pl/m
-                                ) { 
-                                    return $1 
+                                ) {
+                                    return $1
                                 }
-                                
+
                                 return;
                             };
 
@@ -145,12 +144,13 @@ use constant TEST_FAIL_STAGE
 use constant MISSING_PREREQS_LIST
                             => sub {
                                 my $buffer = shift;
+                                my $last = ( split /\[ERROR\] .+? MAKE TEST/, $buffer )[-1];
                                 my @list = map { s/.pm$//; s|/|::|g; $_ }
-                                    ($buffer =~
+                                    ($last =~
                                         m/\bCan\'t locate (\S+) in \@INC/g);
-                                
-                                ### make sure every missing prereq is only 
-                                ### listed ones
+
+                                ### make sure every missing prereq is only
+                                ### listed once
                                 {   my %seen;
                                     @list = grep { !$seen{$_}++ } @list
                                 }
@@ -161,7 +161,7 @@ use constant MISSING_PREREQS_LIST
 use constant MISSING_EXTLIBS_LIST
                             => sub {
                                 my $buffer = shift;
-                                my @list = 
+                                my @list =
                                     ($buffer =~
                                         m/No library found for -l([-\w]+)/g);
 
@@ -174,9 +174,9 @@ use constant REPORT_MESSAGE_HEADER
                                 return << ".";
 
 Dear $author,
-    
+
 This is a computer-generated error report created automatically by
-CPANPLUS, version $version. Testers personal comments may appear 
+CPANPLUS, version $version. Testers personal comments may appear
 at the end of this report.
 
 .
@@ -196,18 +196,35 @@ Below is the error stack from stage '$stage':
 
 $buffer
 
+.
+                            };
+
+use constant REPORT_MESSAGE_PASS_HEADER
+                            => sub {
+                                my($stage, $buffer) = @_;
+                                return << ".";
+
+Thank you for uploading your work to CPAN.  Congratulations!
+All tests were successful.
+
+TEST RESULTS:
+
+Below is the error stack from stage '$stage':
+
+$buffer
+
 .
                             };
 
 use constant REPORT_MISSING_PREREQS
                             => sub {
                                 my ($author,$email,@missing) = @_;
-                                $author = ($author && $email) 
-                                            ? "$author ($email)" 
+                                $author = ($author && $email)
+                                            ? "$author ($email)"
                                             : 'Your Name Here';
-                                
+
                                 my $modules = join "\n", @missing;
-                                my $prereqs = join "\n", 
+                                my $prereqs = join "\n",
                                     map {"\t'$_'\t=> '0',".
                                          " # or a minimum working version"}
                                     @missing;
@@ -231,11 +248,6 @@ $prereqs
     }
 );
 
-If you are interested in making a more flexible Makefile.PL that can
-probe for missing dependencies and install them, ExtUtils::AutoInstall
-at <http://search.cpan.org/dist/ExtUtils-AutoInstall/> may be
-worth a look.
-
 Thanks! :-)
 
 .
@@ -246,7 +258,7 @@ use constant REPORT_MISSING_TESTS
                                 return << ".";
 RECOMMENDATIONS:
 
-It would be very helpful if you could include even a simple test 
+It would be very helpful if you could include even a simple test
 script in the next release, so people can verify which platforms
 can successfully install them, as well as avoid regression bugs?
 
@@ -270,7 +282,7 @@ Thanks!  :-)
 .
                             };
 
-use constant REPORT_LOADED_PREREQS 
+use constant REPORT_LOADED_PREREQS
                             => sub {
                                 my $mod = shift;
                                 my $cb  = $mod->parent;
@@ -278,13 +290,13 @@ use constant REPORT_LOADED_PREREQS
 
                                 ### not every prereq may be coming from CPAN
                                 ### so maybe we wont find it in our module
-                                ### tree at all... 
+                                ### tree at all...
                                 ### skip ones that cant be found in teh list
                                 ### as reported in #12723
                                 my @prq = grep { defined }
                                           map { $cb->module_tree($_) }
                                           sort keys %$prq;
-                                
+
                                 ### no prereqs?
                                 return '' unless @prq;
 
@@ -292,29 +304,85 @@ use constant REPORT_LOADED_PREREQS
                                 my $str = << ".";
 PREREQUISITES:
 
-Here is a list of prerequisites you specified and versions we 
+Here is a list of prerequisites you specified and versions we
 managed to load:
-                                
+
 .
-                                $str .= join '', 
-                                        map { my $want = $prq->{$_->name};
-                                              
-                                              sprintf "\t%s %-30s %8s %8s\n", 
-                                              do { $_->is_uptodate( 
+                                $str .= join '',
+                                        map { sprintf "\t%s %-30s %8s %8s\n",
+                                              @$_
+
+                                        } [' ', 'Module Name', 'Have', 'Want'],
+                                          map { my $want = $prq->{$_->name};
+                                              [ do { $_->is_uptodate(
                                                     version => $want
-                                                   ) ? ' ' : '!' 
-                                              },
-                                              $_->name,
-                                              $_->installed_version,
-                                              $want
-                                              
+                                                   ) ? ' ' : '!'
+                                                },
+                                                $_->name,
+                                                $_->installed_version,
+                                                $want
+                                              ],
                                         ### might be empty entries in there
-                                        } grep { defined $_ } @prq;   
-                                
+                                        } grep { $_ } @prq;
+
                                 return $str;
                             };
 
-use constant REPORT_TESTS_SKIPPED 
+use constant REPORT_TOOLCHAIN_VERSIONS
+                            => sub {
+                                my $mod = shift;
+                                my $cb  = $mod->parent;
+                                #die unless $cb->isa('CPANPLUS::Backend');
+
+                                my @toolchain_modules= qw(
+                                    CPANPLUS
+                                    CPANPLUS::Dist::Build
+                                    Cwd
+                                    ExtUtils::CBuilder
+                                    ExtUtils::Command
+                                    ExtUtils::Install
+                                    ExtUtils::MakeMaker
+                                    ExtUtils::Manifest
+                                    ExtUtils::ParseXS
+                                    File::Spec
+                                    Module::Build
+                                    Pod::Parser
+                                    Pod::Simple
+                                    Test::Harness
+                                    Test::More
+                                    version
+                                );
+
+                                my @toolchain =
+                                          grep { $_ } #module_tree returns '' when module is not found
+                                          map { $cb->module_tree($_) }
+                                          sort @toolchain_modules;
+
+                                ### no prereqs?
+                                return '' unless @toolchain;
+
+                                ### toolchain modules
+                                my $str = << ".";
+
+Perl module toolchain versions installed:
+.
+                                $str .= join '',
+                                        map { sprintf "\t%-30s %8s\n",
+                                              @$_
+
+                                        } ['Module Name', 'Have'],
+                                          map {
+                                              [ $_->name,
+                                                $_->installed_version,
+                                              ],
+                                        ### might be empty entries in there
+                                        } @toolchain;
+
+                                return $str;
+                            };
+
+
+use constant REPORT_TESTS_SKIPPED
                             => sub {
                                 return << ".";
 
@@ -326,7 +394,7 @@ use constant REPORT_TESTS_SKIPPED
 
 .
                             };
-                            
+
 use constant REPORT_MESSAGE_FOOTER
                             => sub {
                                 return << ".";
@@ -334,7 +402,7 @@ use constant REPORT_MESSAGE_FOOTER
 ******************************** NOTE ********************************
 The comments above are created mechanically, possibly without manual
 checking by the sender.  As there are many people performing automatic
-tests on each upload to CPAN, it is likely that you will receive 
+tests on each upload to CPAN, it is likely that you will receive
 identical messages about the same problem.
 
 If you believe that the message is mistaken, please reply to the first
@@ -343,7 +411,7 @@ it personally.  We appreciate your patience. :)
 **********************************************************************
 
 Additional comments:
- 
+
 .
                              };
 
