@@ -13,13 +13,16 @@ use File::Spec;
 use Params::Check               qw[check allow];
 use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 
+use vars qw[$VERSION];
+$VERSION = "0.9134";
+
 $Params::Check::VERBOSE = 1;
 
 =pod
 
 =head1 NAME
 
-CPANPLUS::Internals::Search
+CPANPLUS::Internals::Search - internals for searching for modules
 
 =head1 SYNOPSIS
 
@@ -44,7 +47,7 @@ based on certain criteria and return them.
 
 =head1 METHODS
 
-=head2 _search_module_tree( type => TYPE, allow => \@regexex, [data => \@previous_results ] )
+=head2 _search_module_tree( type => TYPE, allow => \@regexes, [data => \@previous_results ] )
 
 Searches the moduletree for module objects matching the criteria you
 specify. Returns an array ref of module objects on success, and false
@@ -137,13 +140,14 @@ specified in C<data> if provided, rather than the moduletree itself.
 #
 
 sub _search_module_tree {
+
     my $self = shift;
     my $conf = $self->configure_object;
     my %hash = @_;
 
     my($mods,$list,$verbose,$type);
     my $tmpl = {
-        data    => { default    => [values %{$self->module_tree}],
+        data    => { default    => [],
                      strict_type=> 1, store     => \$mods },
         allow   => { required   => 1, default   => [ ], strict_type => 1,
                      store      => \$list },
@@ -153,9 +157,17 @@ sub _search_module_tree {
                      store      => \$type },
     };
 
-    my $args = check( $tmpl, \%hash ) or return;
+    my $args = do {
+        ### don't check the template for sanity
+        ### -- we know it's good and saves a lot of performance
+        local $Params::Check::SANITY_CHECK_TEMPLATE = 0;
 
-    {   local $Params::Check::VERBOSE = 0;
+        check( $tmpl, \%hash );
+    } or return;
+
+    ### a list of module objects was supplied
+    if( @$mods ) {
+        local $Params::Check::VERBOSE = 0;
 
         my @rv;
         for my $mod (@$mods) {
@@ -167,6 +179,13 @@ sub _search_module_tree {
 
         }
         return \@rv;
+
+    } else {
+        my @rv = $self->_source_search_module_tree(
+            allow   => $list,
+            type    => $type,
+        );
+        return \@rv;
     }
 }
 
@@ -214,7 +233,7 @@ sub _search_author_tree {
 
     my($authors,$list,$verbose,$type);
     my $tmpl = {
-        data    => { default    => [values %{$self->author_tree}],
+        data    => { default    => [],
                      strict_type=> 1, store     => \$authors },
         allow   => { required   => 1, default   => [ ], strict_type => 1,
                      store      => \$list },
@@ -226,7 +245,8 @@ sub _search_author_tree {
 
     my $args = check( $tmpl, \%hash ) or return;
 
-    {   local $Params::Check::VERBOSE = 0;
+    if( @$authors ) {
+        local $Params::Check::VERBOSE = 0;
 
         my @rv;
         for my $auth (@$authors) {
@@ -237,9 +257,13 @@ sub _search_author_tree {
             push @rv, $auth if allow( $auth->$type() => $list );
         }
         return \@rv;
+    } else {
+        my @rv = $self->_source_search_author_tree(
+            allow   => $list,
+            type    => $type,
+        );
+        return \@rv;
     }
-
-
 }
 
 =pod
@@ -256,15 +280,19 @@ sub _all_installed {
     my $conf = $self->configure_object;
     my %hash = @_;
 
-    my %seen; my @rv;
-
+    ### File::Find uses follow_skip => 1 by default, which doesn't die
+    ### on duplicates, unless they are directories or symlinks.
+    ### Ticket #29796 shows this code dying on Alien::WxWidgets,
+    ### which uses symlinks.
+    ### File::Find doc says to use follow_skip => 2 to ignore duplicates
+    ### so this will stop it from dying.
+    my %find_args = ( follow_skip => 2 );
 
     ### File::Find uses lstat, which quietly becomes stat on win32
     ### it then uses -l _ which is not allowed by the statbuffer because
     ### you did a stat, not an lstat (duh!). so don't tell win32 to
     ### follow symlinks, as that will break badly
-    my %find_args = ();
-    $find_args{'follow_fast'} = 1 unless $^O eq 'MSWin32';
+    $find_args{'follow_fast'} = 1 unless ON_WIN32;
 
     ### never use the @INC hooks to find installed versions of
     ### modules -- they're just there in case they're not on the
@@ -273,34 +301,56 @@ sub _all_installed {
     ### XXX CPANPLUS::inc is now obsolete, remove the calls
     #local @INC = CPANPLUS::inc->original_inc;
 
+    my %seen; my @rv;
     for my $dir (@INC ) {
         next if $dir eq '.';
 
-        ### not a directory after all ###
+        ### not a directory after all
+        ### may be coderef or some such
         next unless -d $dir;
 
         ### make sure to clean up the directories just in case,
         ### as we're making assumptions about the length
         ### This solves rt.cpan issue #19738
-        $dir = File::Spec->canonpath( $dir );
 
-        File::Find::find(
+        ### John M. notes: On VMS cannonpath can not currently handle
+        ### the $dir values that are in UNIX format.
+        $dir = File::Spec->canonpath( $dir ) unless ON_VMS;
+
+        ### have to use F::S::Unix on VMS, or things will break
+        my $file_spec = ON_VMS ? 'File::Spec::Unix' : 'File::Spec';
+
+        ### XXX in some cases File::Find can actually die!
+        ### so be safe and wrap it in an eval.
+        eval { File::Find::find(
             {   %find_args,
                 wanted      => sub {
 
                     return unless /\.pm$/i;
                     my $mod = $File::Find::name;
 
+                    ### make sure it's in Unix format, as it
+                    ### may be in VMS format on VMS;
+                    $mod = VMS::Filespec::unixify( $mod ) if ON_VMS;
+
                     $mod = substr($mod, length($dir) + 1, -3);
-                    $mod = join '::', File::Spec->splitdir($mod);
+                    $mod = join '::', $file_spec->splitdir($mod);
 
                     return if $seen{$mod}++;
-                    my $modobj = $self->module_tree($mod) or return;
+
+                    my $modobj = $self->module_tree($mod);
+
+                    ### separate return, a list context return with one ''
+                    ### in it, is also true!
+                    return unless $modobj;
 
                     push @rv, $modobj;
                 },
             }, $dir
-        );
+        ) };
+
+        ### report the error if file::find died
+        error(loc("Error finding installed files in '%1': %2", $dir, $@)) if $@;
     }
 
     return \@rv;
