@@ -18,6 +18,9 @@ use Params::Check               qw[check];
 use Module::Load::Conditional   qw[can_load];
 use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 
+use vars qw[$VERSION];
+$VERSION = "0.9134";
+
 $Params::Check::VERBOSE = 1;
 
 ### list of methods the parent class must implement
@@ -551,17 +554,13 @@ sub __create_author_tree {
 
     my ($tot,$prce,$prc,$idx);
 
-    {
+    if ( $args->{verbose} and local $|=1 ) {
       no warnings;
-      $args->{verbose}
-        and local $|=1,
-        $tot = scalar(split /\n/, $cont),
-        ($prce, $prc, $idx) = (int $tot / 25, 0, 0);
+      $tot = scalar(split /\n/, $cont);
+      ($prce, $prc, $idx) = (int $tot / 25, 0, 0);
+      print "\t0%";
     }
 
-    $args->{verbose}
-   and print "\t0%";
-
     for ( split /\n/, $cont ) {
         my($id, $name, $email) = m/^alias \s+
                                     (\S+) \s+
@@ -669,17 +668,13 @@ sub _create_mod_tree {
 
     my($past_header, $count, $tot, $prce, $prc, $idx);
 
-    {
+    if ( $args->{verbose} and local $|=1 ) {
       no warnings;
-      $args->{verbose}
-        and local $|=1,
-        $tot = scalar(split /\n/, $content),
-        ($prce, $prc, $idx) = (int $tot / 25, 0, 0);
+      $tot = scalar(split /\n/, $content);
+      ($prce, $prc, $idx) = (int $tot / 25, 0, 0);
+      print "\t0%";
     }
 
-    $args->{verbose}
-   and print "\t0%";
-
     for ( split /\n/, $content ) {
 
         ### we're still in the header -- find the amount of lines we expect
