@@ -1,14 +1,16 @@
 package CPANPLUS::Module::Author::Fake;
 
-use CPANPLUS::inc;
+
 use CPANPLUS::Module::Author;
 use CPANPLUS::Internals;
 use CPANPLUS::Error;
 
 use strict;
-use vars            qw[@ISA];
+use vars            qw[@ISA $VERSION];
 use Params::Check   qw[check];
 
+$VERSION = "0.9134";
+
 @ISA = qw[CPANPLUS::Module::Author];
 
 $Params::Check::VERBOSE = 1;
@@ -17,12 +19,12 @@ $Params::Check::VERBOSE = 1;
 
 =head1 NAME
 
-CPANPLUS::Module::Author::Fake
+CPANPLUS::Module::Author::Fake - dummy author object for CPANPLUS
 
 =head1 SYNOPSIS
 
     my $auth = CPANPLUS::Module::Author::Fake->new(
-                    name    => 'Foo Bar',
+                    author  => 'Foo Bar',
                     email   => 'luser@foo.com',
                     cpanid  => 'FOO',
                     _id     => $cpan->id,
@@ -50,9 +52,9 @@ sub new {
     my %hash  = @_;
 
     my $tmpl = {
-        author  => { default    => 'CPANPLUS Internals' },
-        email   => { default    => 'cpanplus-info@lists.sf.net' },
-        cpanid  => { default    => 'CPANPLUS' },
+        author  => { default => 'CPANPLUS Internals' },
+        email   => { default => 'cpanplus-info@lists.sf.net' },
+        cpanid  => { default => 'CPANPLUS' },
         _id     => { default => CPANPLUS::Internals->_last_id },
     };
 
@@ -63,7 +65,7 @@ sub new {
     unless( $obj->_id ) {
         error(loc("No '%1' specified -- No CPANPLUS object associated!",'_id'));
         return;
-    } 
+    }
 
     ### rebless object ###
     return bless $obj, $class;
