@@ -1,8 +1,7 @@
 package CPANPLUS::Module::Checksums;
 
 use strict;
-use vars qw[@ISA];
-
+use vars qw[@ISA $VERSION];
 
 use CPANPLUS::Error;
 use CPANPLUS::Internals::Constants;
@@ -16,10 +15,11 @@ use Module::Load::Conditional   qw[can_load];
 $Params::Check::VERBOSE = 1;
 
 @ISA = qw[ CPANPLUS::Module::Signature ];
+$VERSION = "0.9134";
 
 =head1 NAME
 
-CPANPLUS::Module::Checksums
+CPANPLUS::Module::Checksums - checking the checksum of a distribution
 
 =head1 SYNOPSIS
 
@@ -28,7 +28,7 @@ CPANPLUS::Module::Checksums
 
 =head1 DESCRIPTION
 
-This is a class that provides functions for checking the checksum 
+This is a class that provides functions for checking the checksum
 of a distribution. Should not be loaded directly, but used via the
 interface provided via C<CPANPLUS::Module>.
 
@@ -75,7 +75,7 @@ sub _validate_checksum {
 
     ### if we can't check it, we must assume it's ok ###
     return $self->status->checksum_ok(1)
-            unless can_load( modules => { 'Digest::MD5' => '0.0' } );
+            unless can_load( modules => { 'Digest::SHA' => '0.0' } );
     #class CPANPLUS::Module::Status is runtime-generated
 
     my $file = $self->_get_checksums_file( verbose => $verbose ) or (
@@ -102,30 +102,31 @@ sub _validate_checksum {
     } else {
         msg(loc("Archive size is not known for '%1'",$self->package),$verbose);
     }
-    
-    my $md5 = $href->{ $self->package }->{'md5'};
 
-    unless( defined $md5 ) {
-        msg(loc("No 'md5' checksum known for '%1'",$self->package),$verbose);
+    my $sha = $href->{ $self->package }->{'sha256'};
+
+    unless( defined $sha ) {
+        msg(loc("No 'sha256' checksum known for '%1'",$self->package),$verbose);
 
         return $self->status->checksum_ok(1);
     }
 
-    $self->status->checksum_value($md5);
+    $self->status->checksum_value($sha);
 
 
     my $fh = FileHandle->new( $self->status->fetch ) or return;
     binmode $fh;
 
-    my $ctx = Digest::MD5->new;
+    my $ctx = Digest::SHA->new(256);
     $ctx->addfile( $fh );
 
-    my $flag = $ctx->hexdigest eq $md5;
+    my $hexdigest = $ctx->hexdigest;
+    my $flag = $hexdigest eq $sha;
     $flag
         ? msg(loc("Checksum matches for '%1'", $self->package),$verbose)
         : error(loc("Checksum does not match for '%1': " .
-                    "MD5 is '%2' but should be '%3'",
-                    $self->package, $ctx->hexdigest, $md5),$verbose);
+                    "SHA256 is '%2' but should be '%3'",
+                    $self->package, $hexdigest, $sha),$verbose);
 
 
     return $self->status->checksum_ok(1) if $flag;
@@ -141,7 +142,14 @@ sub _get_checksums_file {
     my $clone = $self->clone;
     $clone->package( CHECKSUMS );
 
-    my $file = $clone->fetch( %hash, force => 1 ) or return;
+    # If the user specified a fetchdir, then every CHECKSUMS file will always
+    # be stored there, not in an author-specific subdir.  Thus, in this case,
+    # we need to always re-fetch the CHECKSUMS file and hence need to set the
+    # TTL to something small.
+    my $have_fetchdir =
+        $self->parent->configure_object->get_conf('fetchdir') ne '';
+    my $ttl = $have_fetchdir ? 0.001 : 3600;
+    my $file = $clone->fetch( ttl => $ttl, %hash ) or return;
 
     return $file;
 }
@@ -160,7 +168,7 @@ sub _parse_checksums_file {
 
     ### loop over the header, there might be a pgp signature ###
     my $signed;
-    while (<$fh>) {
+    while (local $_ = <$fh>) {
         last if /^\$cksum = \{\s*$/;    # skip till this line
         my $header = PGP_HEADER;        # but be tolerant of whitespace
         $signed = 1 if /^${header}\s*$/;# due to crossplatform linebreaks
@@ -170,7 +178,7 @@ sub _parse_checksums_file {
     ### *should* be valid perl code
     my $dist;
     my $cksum = {};
-    while (<$fh>) {
+    while (local $_ = <$fh>) {
 
         if (/^\s*'([^']+)' => \{\s*$/) {
             $dist = $1;
@@ -214,7 +222,7 @@ sub _check_signature_for_checksum_file {
     my $fh = OPEN_FILE->($file) or return;
 
     my $signed;
-    while (<$fh>) {
+    while (local $_ = <$fh>) {
         my $header = PGP_HEADER;
         $signed = 1 if /^$header$/;
     }
