@@ -2,12 +2,12 @@ package CPANPLUS::Module::Signature;
 
 use strict;
 
-use CPANPLUS::inc;
 use Cwd;
 use CPANPLUS::Error;
 use Params::Check               qw[check];
 use Module::Load::Conditional   qw[can_load];
-
+use vars qw[$VERSION];
+$VERSION = "0.9134";
 
 ### detached sig, not actually used afaik --kane ###
 #sub get_signature {
