@@ -4,7 +4,7 @@ use strict;
 
 use CPANPLUS::Error;
 use CPANPLUS::Configure;
-
+use CPANPLUS::Internals::Constants;
 
 use Module::Load                qw[load];
 use Params::Check               qw[check];
@@ -12,15 +12,16 @@ use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 
 $Params::Check::VERBOSE = 1;
 
-use vars qw[@ISA $SHELL $DEFAULT];
+use vars qw[@ISA $SHELL $DEFAULT $VERSION];
 
-$DEFAULT    = 'CPANPLUS::Shell::Default';
+$VERSION = "0.9134";
+$DEFAULT    = SHELL_DEFAULT;
 
 =pod
 
 =head1 NAME
 
-CPANPLUS::Shell
+CPANPLUS::Shell - base class for CPANPLUS shells
 
 =head1 SYNOPSIS
 
@@ -49,18 +50,23 @@ choice.
 
 =cut
 
-
 sub import {
     my $class   = shift;
     my $option  = shift;
-    ### XXX this should offer to reconfigure CPANPLUS, somehow.  --rs
-    my $conf    = CPANPLUS::Configure->new() 
-                    or die loc("No configuration available -- aborting") . $/;
 
     ### find out what shell we're supposed to load ###
     $SHELL      = $option
                     ? $class . '::' . $option
-                    : $conf->get_conf('shell') || $DEFAULT;
+                    : do {  ### XXX this should offer to reconfigure
+                            ### CPANPLUS, somehow.  --rs
+                            ### XXX load Configure only if we really have to
+                            ### as that means any $Conf passed later on will
+                            ### be ignored in favour of the one that was
+                            ### retrieved via ->new --kane
+                        my $conf = CPANPLUS::Configure->new() or
+                        die loc("No configuration available -- aborting") . $/;
+                        $conf->get_conf('shell') || $DEFAULT;
+                    };
 
     ### load the shell, fall back to the default if required
     ### and die if even that doesn't work
@@ -119,6 +125,8 @@ $TMPL = {
     remote          => { default => undef },
     noninteractive  => { default => '' },
     cache           => { default => [ ] },
+    settings        => { default => { install_all_prereqs => undef },
+                         no_override => 1 },
     _old_sigpipe    => { default => '', no_override => 1 },
     _old_outfh      => { default => '', no_override => 1 },
     _signals        => { default => { INT => { } }, no_override => 1 },
@@ -185,11 +193,13 @@ sub _show_banner {
     $rl_avail = loc("ReadLine support %1.", $rl_avail);
     $rl_avail = "\n*** $rl_avail" if (length($rl_avail) > 45);
 
-    print loc("%1 -- CPAN exploration and module installation (v%2)",
+    $self->__print(
+          loc("%1 -- CPAN exploration and module installation (v%2)",
                 $self->which, $self->which->VERSION()), "\n",
           loc("*** Please report bugs to <bug-cpanplus\@rt.cpan.org>."), "\n",
           loc("*** Using CPANPLUS::Backend v%1.  %2",
-                $cpan->VERSION, $rl_avail), "\n\n";
+                $cpan->VERSION, $rl_avail), "\n\n"
+    );
 }
 
 ### checks whether the Term::ReadLine is broken and needs to fallback to Stub
@@ -279,6 +289,24 @@ sub _pager_close {
     }
 }
 
+### Custom print routines, mainly to be able to catch output
+### in test cases, or redirect it if need be
+{   sub __print {
+        my $self = shift;
+        print @_;
+    }
+
+    sub __printf {
+        my $self = shift;
+        my $fmt  = shift;
+
+        ### MUST specify $fmt as a separate param, and not as part
+        ### of @_, as it will then miss the $fmt and return the
+        ### number of elements in the list... =/ --kane
+        $self->__print( sprintf( $fmt, @_ ) );
+    }
+}
+
 1;
 
 =pod
@@ -293,10 +321,10 @@ This module by Jos Boumans E<lt>kane@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-The CPAN++ interface (of which this module is a part of) is copyright (c) 
+The CPAN++ interface (of which this module is a part of) is copyright (c)
 2001 - 2007, Jos Boumans E<lt>kane@cpan.orgE<gt>. All rights reserved.
 
-This library is free software; you may redistribute and/or modify it 
+This library is free software; you may redistribute and/or modify it
 under the same terms as Perl itself.
 
 =head1 SEE ALSO
