@@ -8,7 +8,7 @@ package CPANPLUS::Shell::Classic;
 
 use strict;
 
-use CPANPLUS::inc;
+
 use CPANPLUS::Error;
 use CPANPLUS::Backend;
 use CPANPLUS::Configure::Setup;
@@ -30,7 +30,7 @@ $Params::Check::ALLOW_UNKNOWN = 1;
 BEGIN {
     use vars        qw[ $VERSION @ISA ];
     @ISA        =   qw[ CPANPLUS::Shell::_Base::ReadLine ];
-    $VERSION    =   '0.0562';
+    $VERSION = "0.9134";
 }
 
 load CPANPLUS::Shell;
@@ -97,6 +97,20 @@ sub new {
             code    => \&__ask_about_test_report,
     );
 
+    if (my $histfile = $self->configure_object->get_conf( 'histfile' )) {
+        my $term = $self->term;
+        if ($term->can('AddHistory')) {
+            if (open my $fh, '<', $histfile) {
+                local $/ = "\n";
+                while (my $line = <$fh>) {
+                    chomp($line);
+                    $term->AddHistory($line);
+                }
+                close($fh);
+            }
+        }
+    }
+
     return $self;
 }
 
@@ -153,8 +167,6 @@ sub _dispatch_on_input {
     ### the original force setting;
     my $force_store = $conf->get_conf( 'force' );
 
-
-    my $key;
     ### parse the input: the first part before the space
     ### is the command, followed by arguments.
     ### see the usage below
@@ -196,6 +208,24 @@ sub _dispatch_on_input {
 
 ### displays quit message
 sub _quit {
+    my $self = shift;
+    my $term = $self->term;
+
+    if ($term->can('GetHistory')) {
+        my @history = $term->GetHistory;
+
+        my $histfile = $self->configure_object->get_conf('histfile');
+
+        if (open my $fh, '>', $histfile) {
+            foreach my $line (@history) {
+                print {$fh} "$line\n";
+            }
+            close($fh);
+        }
+        else {
+            warn "Cannot open history file '$histfile' - $!";
+        }
+    }
 
     ### well, that's what CPAN.pm says...
     print "Lockfile removed\n";
@@ -316,7 +346,7 @@ sub _shell {
 
         $cb->_chdir( dir => $obj->status->extract )   or next;
 
-        local $ENV{PERL5OPT} = CPANPLUS::inc->original_perl5opt;
+        #local $ENV{PERL5OPT} = CPANPLUS::inc->original_perl5opt;
         if( system($shell) and $! ) {
             print "Error executing your subshell '$shell': $!\n";
             next;
@@ -1199,20 +1229,28 @@ using CPANPLUS underneath.
 
 For detailed documentation, refer to L<CPAN>.
 
-=head1 AUTHORS
+=head1 BUG REPORTS
 
-This module by
-Jos Boumans E<lt>kane@cpan.orgE<gt>.
+Please report bugs or other issues to E<lt>bug-cpanplus@rt.cpan.org<gt>.
+
+=head1 AUTHOR
+
+This module by Jos Boumans E<lt>kane@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-The CPANPLUS interface (of which this module is a part of) is
-copyright (c) 2001-2004 Jos Boumans E<lt>kane@cpan.orgE<gt>.
-All rights reserved.
+The CPAN++ interface (of which this module is a part of) is copyright (c)
+2001 - 2007, Jos Boumans E<lt>kane@cpan.orgE<gt>. All rights reserved.
+
+This library is free software; you may redistribute and/or modify it
+under the same terms as Perl itself.
+
+=head1 SEE ALSO
+
+L<CPANPLUS::Configure>, L<CPANPLUS::Module>, L<CPANPLUS::Module::Author>
+
+=cut
 
-This library is free software;
-you may redistribute and/or modify it under the same
-terms as Perl itself.
 
 =head1 SEE ALSO
 
