@@ -7,9 +7,12 @@ use Params::Check               qw[check];
 use CPANPLUS::Error             qw[error msg];
 use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 
+use vars qw[$VERSION];
+$VERSION = "0.9134";
+
 =head1 NAME
 
-CPANPLUS::Shell::Default::Plugins::Remote
+CPANPLUS::Shell::Default::Plugins::Remote - connect to a remote CPANPLUS
 
 =head1 SYNOPSIS
 
@@ -29,17 +32,17 @@ like this:
     CPAN Terminal> /connect --user=my_user --pass=secret localhost 1337
 
     Connection accepted
-    
+
     Successfully connected to 'localhost' on port '11337'
-    
+
     Note that no output will appear until a command has completed
     -- this may take a while
 
 
     CPAN Terminal@localhost> o; i *
-    
+
     [....]
-    
+
     CPAN Terminal@localhost> /disconnect
 
     CPAN Terminal>
@@ -64,10 +67,8 @@ sub connect {
     {   local $Params::Check::ALLOW_UNKNOWN = 1;
 
         my $tmpl = {
-            user => { default => $conf->_get_daemon('username'),
-                        store => \$user },
-            pass => { default => $conf->_get_daemon('password'),
-                        store => \$pass },
+            user => { default   => 'cpanpd',    store => \$user },
+            pass => { required  => 1,           store => \$pass },
         };
 
          check( $tmpl, $opts ) or return;
@@ -75,7 +76,7 @@ sub connect {
 
     my @parts = split /\s+/, $input;
     my $host = shift @parts || 'localhost';
-    my $port = shift @parts || $conf->_get_daemon('port');
+    my $port = shift @parts || '1337';
 
     load IO::Socket;
 
@@ -114,7 +115,7 @@ sub connect {
         ### save the original prompt
         $Saved_Prompt = $shell->prompt;
 
-        $shell->prompt( $shell->brand .'@'. $host .'> ' );
+        $shell->prompt( $shell->brand .'@'. $host .':'. $port .'> ' );
 
     } else {
         print "\n$buffer\n\n";
@@ -141,7 +142,7 @@ sub disconnect {
 }
 
 sub connect_help {
-    return loc( 
+    return loc(
             "    /connect [HOST PORT]   # Connect to the remote machine,\n" .
             "                           # defaults taken from your config\n" .
             "        --user=USER        # Optional username\n" .
@@ -153,8 +154,8 @@ sub disconnect_help {
             "    /disconnect            # Disconnect from the remote server" );
 }
 
-1; 
-        
+1;
+
 =pod
 
 =head1 BUG REPORTS
@@ -167,10 +168,10 @@ This module by Jos Boumans E<lt>kane@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-The CPAN++ interface (of which this module is a part of) is copyright (c) 
+The CPAN++ interface (of which this module is a part of) is copyright (c)
 2001 - 2007, Jos Boumans E<lt>kane@cpan.orgE<gt>. All rights reserved.
 
-This library is free software; you may redistribute and/or modify it 
+This library is free software; you may redistribute and/or modify it
 under the same terms as Perl itself.
 
 =head1 SEE ALSO
